#include <AppFileInfo.h>
#include <Resources.h>
#include <MenuItem.h>
#include <NodeInfo.h>
#include <Deskbar.h>
#include <Message.h>
#include <Bitmap.h>
#include <Roster.h>
#include <String.h>
#include <Roster.h>
#include <Alert.h>
#include <Entry.h>
#include <string>

#include "LinkSysWindow.h"
#include "DeskbarView.h"
#include "Messages.h"
#include "Globals.h"

extern "C" _EXPORT BView *instantiate_deskbar_item();
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BBitmap *get_resource_bitmap()
{
  entry_ref	ref;
  BBitmap *bmp = new	BBitmap(BRect(0,0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);	
  if (be_roster->FindApp(GLOBAL_APP_SIG, &ref)!=B_OK) return NULL;
  BNode      file( &ref );
  BNodeInfo  *info = new BNodeInfo(&file);
  info->GetIcon(bmp, B_MINI_ICON);
  delete info;
  return bmp;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BView *instantiate_deskbar_item()
{ return new DeskbarView(); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
DeskbarView::DeskbarView()
	:	BView( BRect(0, 0, 15, 15), VIEW_NAME, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_PULSE_NEEDED)
{
	SetViewColor(186,186,186);
	SetDrawingMode(B_OP_OVER);
	fBitmap = get_resource_bitmap();
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
DeskbarView::DeskbarView(BMessage *archive)
	: BView(archive)
{
	SetViewColor(186,186,186);
	SetDrawingMode(B_OP_OVER);
	fBitmap = get_resource_bitmap();
    window = new LinkSysWindow();
    endpoint = new BNetEndpoint(SOCK_DGRAM);   
    menu = new BPopUpMenu("Quit Menu");
    menu->AddItem( new BMenuItem( "LinkSys Log Viewer", new BMessage(ABOUT) ) );
    menu->AddSeparatorItem();
    menu->AddItem( new BMenuItem( "Quit", new BMessage(B_QUIT_REQUESTED)) );

  if( B_ERROR == endpoint->Bind( 162 ) ) {
    cout << "ERROR Binding Port" << endl << "Make sure no other SNMP programs are running" << endl;
    BAlert *Alert = new BAlert("Error Binding SNMP port", "Unable to open SNMP port for listening. Another copy of LinkSys Log Viewer may already be running.\n\nYou may also want to try restartng Networking.", "DO'H!", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
    Alert->Go(NULL);
    RemoveFromDeskbar();
  }
  if(B_OK == endpoint->Listen() ) { cout << "Bound Port 162... Listening" << endl; }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::Pulse()
{
  BString buffer;
  BMessenger msngr = BMessenger(NULL, window);
  bool inbound;
  int bytesReceived = endpoint->Receive(buffer.LockBuffer(255), 255);
  if( bytesReceived == -1 )
  {
    endpoint->Close();
    endpoint->Bind( 162 );
  } else {
    buffer.UnlockBuffer( bytesReceived );
    if( buffer.Length() > 74 ) {
      buffer.Remove(0,74);
      BMessage *msg = new BMessage( HIT );
      msg->AddInt32( "time", real_time_clock() );
      msg->AddString( "string", buffer );
     msngr.SendMessage( msg, msg );
      msg->FindBool("inbound", &inbound);
      if( inbound )
        SetViewColor(255,0,0);
      else
        SetViewColor(0,252,0);
      delete msg;  
    } else {
      SetViewColor(186,186,186);
    }
    Invalidate();
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::Draw(BRect rect)
{ DrawBitmap( fBitmap ); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::AddToDeskbar()
{
	BDeskbar *deskbar = new BDeskbar();
	entry_ref ref;
	be_roster->FindApp(GLOBAL_APP_SIG, &ref);
	deskbar->AddItem(&ref);
	delete deskbar;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::RemoveFromDeskbar() {
    window->Lock();
    window->Hide();
    window->Quit();
    delete fBitmap;
    delete endpoint;
    delete menu;
	BDeskbar deskbar;
	deskbar.RemoveItem(VIEW_NAME);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::MouseDown(BPoint p)
{
  BPoint cursor;
  uint32 buttons;
  GetMouse(&cursor, &buttons);
  if(B_SECONDARY_MOUSE_BUTTON == buttons)
  {
    ConvertToScreen(&p);
    BMenuItem *menuItem = menu->Go(p);
    if(  NULL != menuItem )
    {
      if( 0 == strcmp(menuItem->Label(),"Quit") )
        RemoveFromDeskbar();
      else if( 0 == strcmp(menuItem->Label(),"LinkSys Log Viewer") )
        window->PostMessage(ABOUT);
    }
  } else {
    window->SetWorkspaces(B_CURRENT_WORKSPACE);
    window->Activate(true);
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BArchivable *DeskbarView::Instantiate(BMessage *data)
{ return new DeskbarView(data); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
status_t DeskbarView::Archive(BMessage *data, bool deep) const
{
	BView::Archive(data, deep);
	data->AddString("add_on", GLOBAL_APP_SIG);
	data->AddString("class", VIEW_NAME);
	return B_OK;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
